/* -*- mode: c ; c-file-style: "canonware-c-style" -*-
 ******************************************************************************
 *
 * Copyright (C) 1996-2004 Jason Evans <jasone@canonware.com>.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice(s), this list of conditions and the following disclaimer
 *    unmodified other than the allowable addition of one or more
 *    copyright notices.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice(s), this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER(S) ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************
 *
 * Version: Onyx 5.1.1
 *
 ******************************************************************************/

#include "../include/libonyx/libonyx.h"

#include <sys/time.h>
#include <errno.h>

void
mtx_new(cw_mtx_t *a_mtx)
{
#ifdef CW_PTHREADS
    int error;
#endif

    cw_check_ptr(a_mtx);

#ifdef CW_PTH
    if (pth_mutex_init(&a_mtx->mutex) == false)
    {
	fprintf(stderr, "%s:%d:%s(): Error in pth_mutex_init: %s\n",
		__FILE__, __LINE__, __func__, strerror(errno));
	abort();
    }
#endif
#ifdef CW_PTHREADS
    error = pthread_mutex_init(&a_mtx->mutex, NULL);
    if (error)
    {
	fprintf(stderr, "%s:%d:%s(): Error in pthread_mutex_init: %s\n",
		__FILE__, __LINE__, __func__, strerror(error));
	abort();
    }
#endif
}

void
mtx_delete(cw_mtx_t *a_mtx)
{
#ifdef CW_PTHREADS
    int error;
#endif

    cw_check_ptr(a_mtx);

#ifdef CW_PTHREADS
    error = pthread_mutex_destroy(&a_mtx->mutex);
    if (error)
    {
	fprintf(stderr, "%s:%d:%s(): Error in pthread_mutex_destroy(): %s\n",
		__FILE__, __LINE__, __func__, strerror(error));
	abort();
    }
#endif
}

void
mtx_lock(cw_mtx_t *a_mtx)
{
#ifdef CW_PTHREADS
    int error;
#endif

    cw_check_ptr(a_mtx);

#ifdef CW_PTH
    if (pth_mutex_acquire(&a_mtx->mutex, false, NULL) == false)
    {
	fprintf(stderr, "%s:%d:%s(): Error in pth_mutex_acquire(): %s\n",
		__FILE__, __LINE__, __func__, strerror(errno));
	abort();
    }
#endif
#ifdef CW_PTHREADS
    error = pthread_mutex_lock(&a_mtx->mutex);
    if (error)
    {
	fprintf(stderr, "%s:%d:%s(): Error in pthread_mutex_lock(): %s\n",
		__FILE__, __LINE__, __func__, strerror(error));
	abort();
    }
#endif
}

bool
mtx_trylock(cw_mtx_t *a_mtx)
{
    bool retval;
#ifdef CW_PTHREADS
    int error;
#endif

    cw_check_ptr(a_mtx);

#ifdef CW_PTH
    if (pth_mutex_acquire(&a_mtx->mutex, true, NULL))
    {
	retval = false;
    }
    else if (errno == EBUSY)
    {
	retval = true;
    }
    else
    {
	fprintf(stderr, "%s:%d:%s(): Error in pth_mutex_acquire(): %s\n",
		__FILE__, __LINE__, __func__, strerror(errno));
	abort();
    }
#endif
#ifdef CW_PTHREADS
    error = pthread_mutex_trylock(&a_mtx->mutex);
    if (error == 0)
    {
	retval = false;
    }
    else if (error == EBUSY)
    {
	retval = true;
    }
    else
    {
	fprintf(stderr, "%s:%d:%s(): Error in pthread_mutex_trylock(): %s\n",
		__FILE__, __LINE__, __func__, strerror(error));
	abort();
    }
#endif

    return retval;
}

void
mtx_unlock(cw_mtx_t *a_mtx)
{
#ifdef CW_PTHREADS
    int error;
#endif

    cw_check_ptr(a_mtx);

#ifdef CW_PTH
    if (pth_mutex_release(&a_mtx->mutex) == false)
    {
	fprintf(stderr, "%s:%d:%s(): Error in pth_mutex_release(): %s\n",
		__FILE__, __LINE__, __func__, strerror(errno));
	abort();
    }
#endif
#ifdef CW_PTHREADS
    error = pthread_mutex_unlock(&a_mtx->mutex);
    if (error)
    {
	fprintf(stderr, "%s:%d:%s(): Error in pthread_mutex_unlock(): %s\n",
		__FILE__, __LINE__, __func__, strerror(error));
	abort();
    }
#endif
}
