/* -*- mode: c ; c-file-style: "canonware-c-style" -*-
 ******************************************************************************
 *
 * Copyright (C) 1996-2004 Jason Evans <jasone@canonware.com>.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice(s), this list of conditions and the following disclaimer
 *    unmodified other than the allowable addition of one or more
 *    copyright notices.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice(s), this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER(S) ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************
 *
 * Version: Onyx 5.1.1
 *
 * thd test.
 *
 ******************************************************************************/

#include "../include/libonyx/libonyx.h"

cw_mtx_t mtx;
cw_thd_t *thread_a, *thread_b;

void *
thread_entry_func(void *a_arg)
{
    char *arg_str = (char *) a_arg;

    fprintf(stderr, "%s(): Argument string: \"%s\"\n", __func__, arg_str);

    mtx_lock(&mtx);
    fprintf(stderr, "%s(): thd_self() returns %s\n", __func__,
	    (thd_self() == thread_a) ? "thread_a" : (thd_self() == thread_b)
	    ? "thread_b" : "<error>");
    mtx_unlock(&mtx);

    return NULL;
}

int
main()
{
    libonyx_init(0, NULL, NULL);
    fprintf(stderr, "Test begin\n");
    mtx_new(&mtx);

    thread_b = NULL;
    mtx_lock(&mtx);
    thread_a = thd_new(thread_entry_func, (void *)"Thread A argument", true);
    mtx_unlock(&mtx);
    cw_assert(thd_join(thread_a) == NULL);
    thread_a = NULL;

    mtx_lock(&mtx);
    thread_b = thd_new(thread_entry_func, (void *)"Thread B argument", true);
    mtx_unlock(&mtx);
    cw_assert(thd_join(thread_b) == NULL);

    mtx_delete(&mtx);
    fprintf(stderr, "Test end\n");
    libonyx_shutdown();

    return 0;
}
